<template>
  <Dialog v-model="dialogVisible" :title="dialogTitle" width="50%">
    <el-form
      ref="formRef"
      v-loading="formLoading"
      :model="formData"
      :rules="formRules"
      label-width="120px"
    >
      <el-row>
        <el-col :span="12">
          <el-form-item label="服务名" prop="serverName">
            <el-input v-model="formData.serverName" placeholder="请输入服务名"/>
          </el-form-item>
        </el-col>
        <el-col :span="12">
          <el-form-item label="端点URL" prop="endpointUrl">
            <el-input v-model="formData.endpointUrl" placeholder="请输入端点URL"/>
          </el-form-item>
        </el-col>
      </el-row>
      <el-row>
        <el-col :span="12">
          <el-form-item label="安全策略" prop="securityPolicy">
            <el-select v-model="formData.securityPolicy" placeholder="请选择">
              <el-option
                v-for="dict in getStrDictOptions(DICT_TYPE.OPCUA_SECURITY_POLICY)"
                :key="dict.value"
                :label="dict.label"
                :value="dict.value"
              />
            </el-select>
          </el-form-item>
        </el-col>
        <el-col :span="12">
          <el-form-item label="安全模式" prop="securityMode">
            <el-select v-model="formData.securityMode" placeholder="请选择">
              <el-option
                v-for="dict in getStrDictOptions(DICT_TYPE.OPCUA_SECURITY_MODE)"
                :key="dict.value"
                :label="dict.label"
                :value="dict.value"
              />
            </el-select>
          </el-form-item>
        </el-col>
      </el-row>
      <el-row>
        <el-col :span="12">
          <el-form-item label="连接方式" prop="connectionType">
            <el-select v-model="formData.connectionType" placeholder="请选择">
              <el-option
                v-for="dict in getStrDictOptions(DICT_TYPE.OPCUA_CONNECTION_TYPE)"
                :key="dict.value"
                :label="dict.label"
                :value="dict.value"
              />
            </el-select>
          </el-form-item>
        </el-col>
      </el-row>
      <el-row>
        <el-col :span="12">
          <el-form-item label="用户名" prop="userName">
            <el-input v-model="formData.userName" placeholder="请输入用户名"/>
          </el-form-item>
        </el-col>
        <el-col :span="12">
          <el-form-item label="密码" prop="password">
            <el-input type="password" v-model="formData.password" placeholder="请输入密码"/>
          </el-form-item>
        </el-col>
      </el-row>
      <el-row>
        <el-col :span="24">
          <el-form-item label="安全证书路径" prop="certificatePath">
            <el-input v-model="formData.certificatePath" placeholder="请输入安全证书路径"/>
          </el-form-item>
        </el-col>
      </el-row>
      <el-row>
        <el-col :span="12">
          <el-form-item label="不活动超时" prop="connectInactivityTimeout">
            <el-input v-model="formData.connectInactivityTimeout" placeholder="请输入不活动超时"/>
          </el-form-item>
        </el-col>
        <el-col :span="12">
          <el-form-item label="重连超时" prop="reconnectInterval">
            <el-input v-model="formData.reconnectInterval" placeholder="请输入重连超时"/>
          </el-form-item>
        </el-col>
      </el-row>
    </el-form>
    <template #footer>
      <el-button :disabled="formLoading" type="primary" @click="submitForm">确 定</el-button>
      <el-button @click="dialogVisible = false">取 消</el-button>
    </template>
  </Dialog>
</template>
<script lang="ts" setup>
import * as OpcUaApi from '@/api/data/channel/opcua'
import { DICT_TYPE, getStrDictOptions } from '@/utils/dict'

defineOptions({name: 'DataOpcUaDeviceForm'})

const {t} = useI18n() // 国际化
const message = useMessage() // 消息弹窗
const dialogVisible = ref(false) // 弹窗的是否展示
const dialogTitle = ref('') // 弹窗的标题
const formLoading = ref(false) // 表单的加载中：1）修改时的数据加载；2）提交的按钮禁用
const formType = ref('') // 表单的类型：create - 新增；update - 修改
const formData = ref({
  id: undefined,
  serverName: undefined,
  endpointUrl: undefined,
  securityPolicy: undefined,
  securityMode: undefined,
  connectionType: undefined,
  userName: undefined,
  password: undefined,
  certificatePath: undefined,
  connectInactivityTimeout: undefined,
  reconnectInterval: 5000,
})
const formRules = reactive({
  serverName: [{required: true, message: '服务名不能为空', trigger: 'blur'}],
  endpointUrl: [{required: true, message: '端点URL不能为空', trigger: 'blur'}],
  securityPolicy: [{required: true, message: '安全策略不能为空', trigger: 'blur'}],
  securityMode: [{required: true, message: '安全模式不能为空', trigger: 'blur'}],
  connectionType: [{required: true, message: '连接方式不能为空', trigger: 'blur'}],
})
const formRef = ref() // 表单 Ref

/** 打开弹窗 */
const open = async (type: string, id?: number) => {
  dialogVisible.value = true
  dialogTitle.value = t('action.' + type)
  formType.value = type
  resetForm()
  // 修改时，设置数据
  if (id) {
    formLoading.value = true
    try {
      formData.value = await OpcUaApi.getOpcUaDevice(id)
    } finally {
      formLoading.value = false
    }
  }
}
defineExpose({open}) // 提供 open 方法，用于打开弹窗

/** 提交表单 */
const emit = defineEmits(['success']) // 定义 success 事件，用于操作成功后的回调
const submitForm = async () => {
  // 校验表单
  if (!formRef) return
  const valid = await formRef.value.validate()
  if (!valid) return
  // 提交请求
  formLoading.value = true
  try {
    const data = formData.value as unknown as OpcUaApi.OpcUaDeviceVO
    if (formType.value === 'create') {
      await OpcUaApi.createOpcUaDevice(data)
      message.success(t('common.createSuccess'))
    } else {
      await OpcUaApi.updateOpcUaDevice(data)
      message.success(t('common.updateSuccess'))
    }
    dialogVisible.value = false
    // 发送操作成功的事件
    emit('success')
  } finally {
    formLoading.value = false
  }
}

/** 重置表单 */
const resetForm = () => {
  formData.value = {
    id: undefined,
    serverName: undefined,
    endpointUrl: undefined,
    securityPolicy: undefined,
    securityMode: undefined,
    connectionType: undefined,
    userName: undefined,
    password: undefined,
    certificatePath: undefined,
    connectInactivityTimeout: undefined,
    reconnectInterval: 5000,
  }
  formRef.value?.resetFields()
}
</script>
